(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
(*  Soubor: AKUTEST.PAS                                                    *)
(*  Obsah: ridici program pro merak vybijecich charakteristik akumulatoru  *)
(*  Autor: Mircosoft (http://mircosoft.mzf.cz)                             *)
(*  Posledni uprava: 3.2.2024                                              *)
(*  Pro kompilaci: viz Uses                                                *)
(*  Pro spusteni: nic (vytvori se AKUTEST.CFG)                             *)
(*  Upozorneni: tyto zdrojove kody pouzivate na vlastni nebezpeci          *)
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
program merak_baterii;
{$I-,N+,E-}
uses klavesy2,texty2,textmenu,retezce,matyka,cfg,dos;

{konfigurovatelne provozni parametry:}
const CekaniNaPrevod:byte=200;
      CekaniNaCS:byte=100;
      CekaniClkHigh:byte=100;
      CekaniClkLow:byte=100;
      CisloPortu:byte=0; {1..4}
      BA:word=0; {bazova adresa portu, se kterym pracujeme}
      SpousteciNapeti:real=16;{[V]}
      VypinaciNapeti:real=9;{[V]}
      IntervalZaznamu:word=10;{[s]}
      JmenoZaznamu:string[79]='ZAZNAM.CSV'; {normalne by to byl pathstr, delku pisu pro jistotu kvuli zadavani do konfigurace}
      OddelovacSloupcu:char=','; {\  Moznost prenastaveni je tu jenom kvuli }
      OddelovacDesetin:char='.'; { \ Excelu 2007 a novejsim, ktere pouzivaji}
      Uvozovka:string[1]='';     { / nestandardne strednik, carku a nekdy i }
      RozsahUA:word=5000;{[mV]}  {   uvozovky.                              }
      RozsahUB:word=15000;{[mV]}
      RozsahIA:word=1000;{[mA]}
      RozsahIB:word=3000;{[mA]}
      PocetVzorku:byte=1;{kolik vzorku se ma namerit a zprumerovat (1..100)}
{Nejmensi dilek nameritelny osmibitovym A/D prevodnikem:
U: 19.5 mV pri rozsahu 5 V, 58.6 mV pri rozsahu 15 V
I: 3.9 mA pri rozsahu 1 A, 11.7 mA pri rozsahu 3 A
Jak je videt, specifikace rozsahu s presnosti na mili bohate staci.}

{pracovni promenne:}
type TypRezimu=(mHlavniMenu,mVyberPortu,mKalibrace,
                mRucniMereni,mAutomatickeMereni,mNastaveni,mKonec);

var BazoveAdresy:array[1..4] of word absolute 0:$0400; {adresy portu ulozene
     v BIOSu, obvykle $3F8,$2F8,$3E8 a $2E8, neexistujici jsou obvykle nulove}
    rezim:typrezimu; {pro rizeni chodu programu}

{procedury pro obsluhu hardwaru:}

function JeRozsahNapetiB:boolean; {false=A (5 V), true=B (15 V)}
Begin jerozsahnapetib:=(port[ba+6] shr 6) and 1<>0; {RI} End;

function JeRozsahProuduB:boolean; {false=A (1 A), true=B (3 A)}
Begin jerozsahproudub:=(port[ba+6] shr 7) and 1<>0; {DCD} End;

procedure Rele(zapnout:boolean); {false=vypnuto, true=zapnuto}
Begin if zapnout then port[ba+3]:=$40 else port[ba+3]:=0; {TXD} End;

procedure CtiAD(var U,I:byte); {nacte aktualni vystupy z obou prevodniku (syrove 0..255)}
var cekac,IndexBitu:byte;
Begin
asm cli end; {zakaz preruseni, aby nam neco nerozhodilo cekaci smycky (viz dale poznamku 2)}
Port[ba+4]:=1; {zapni DTR (pin 5 - Chip Select), tim se spusti prevod
posledne naakumulovaneho vstupniho napeti na 8bitove cislo}
for cekac:=1 to cekaninaprevod do asm nop end; {cekani min. 17 mikrosekund,
nez se prevod dokonci (systemovy casovac s nekolikamilisekundovym rozlisenim
nema cenu zkouset, potrebujeme mikrosekundy)}
Port[ba+4]:=0; {vypni DTR, tim se vysledek presune do vystupniho registru,
cip se prepne do rezimu odesilani a na vystupu Data Out (CTS) se objevi
nejvyssi bit vysledku}
for cekac:=1 to cekaninacs do asm nop end; {cekani min. 1.4 mikrosekundy, nez
se operace dokonci (spolecne s instrukcemi na nasledujicich radcich to tu
mikrosekundu da)}
U:=0; I:=0;
for indexbitu:=1 to 8 do
 begin
 {ulozime bity z linek CTS a DSR (pin 6 - Data Out):}
 U:=(U shl 1) or ((port[ba+6] shr 4) and 1);
 I:=(I shl 1) or ((port[ba+6] shr 5) and 1);
 {a blikneme linkou RTS (pin 7 - I/O Clock), po jeji sestupne hrane se na
 vystupech objevi dalsi bit:}
 Port[ba+4]:=2; {zapni RTS}
 for cekac:=1 to cekaniclkhigh do asm nop end; {cekani min. 404 nanosekund}
 Port[ba+4]:=0; {vypni RTS}
 for cekac:=1 to cekaniclklow do asm nop end; {Vetsinu z predepsanych 404 ns
 s vypnutou RTS zabere forcyklus pro bity, tohle posledni cekani je jenom
 pojistka pro extremne rychle procesory.}
 {Uprostred cteni (po ctvrte sestupne hrane RTS) si cip zacne
 akumulovat nove vstupni napeti pro priste.}
 end;
asm sti end; {zase povol preruseni}
{Pozn. c. 1: Mereni a odesilani probiha paralelne, takze hodnota, kterou jste
z cipu prave precetli, byla namerena uz v okamziku minuleho cteni. Na to pozor
hlavne v pripadech, kdy na okamziku dosazeni urciteho napeti zavisi spusteni
nejake akce (treba konec vybijeni akumulatoru)!
Pozn. c. 2: Maximalni cekaci doby mezi ridicimi impulsy datasheet neuvadi,
ale neomezene nejsou. Kdyz jsou moc dlouhe (napr. kdyz se pokousite cip
ovladat rucne), dostanete vzdy vysledek 255 ($FF).}
End;{ctiAD}

procedure ZmerHodnoty(var U,I:real; var Ub,Ib:byte);
{Mereni napeti a proudu s prumerovanim.
 U,I - napeti ve V a proud v A (vystup)
 Ub,Ib - syrove napeti a proud primo z prevodniku (0..255) (vystup)}
var bU,bI,pocitadlo,pojistka:byte;
    soucetU,soucetI:word;
    PRU,PRI:boolean; {rozsahy napeti a proudu pred zacatkem mereni}
    rozsah:real; {teoreticky by stacil word, ale pak by se ve vypoctu stejne musel pretypovat na real, aby nepretekl}
Begin
ctiad(bu,bi); {opozdene hodnoty z minuleho mereni precteme a zahodime}
pojistka:=0;
 repeat
 pru:=jerozsahnapetib; pri:=jerozsahproudub;
 soucetu:=0; souceti:=0;
 for pocitadlo:=1 to pocetvzorku do begin
                                    ctiad(bu,bi);
                                    inc(soucetu,bu);
                                    inc(souceti,bi);
                                    end;
 if (jerozsahnapetib=pru)and(jerozsahproudub=pri)
   then begin {zprumerovani namerenych vzorku a prepocet na volty a ampery}
        if pru then rozsah:=rozsahUB else rozsah:=rozsahUA;
        U:=(rozsah*soucetU)/(256000*pocetvzorku); {256 je rozsah cisla z ADC,
                1000 je pro prevod milivoltu (rozsah napeti je v mV) na volty}
        Ub:=soucetU div pocetvzorku; {tohle uz je jenom prumer bez prepoctu}
        if pri then rozsah:=rozsahIB else rozsah:=rozsahIA;
        I:=(rozsah*soucetI)/(256000*pocetvzorku);
        Ib:=soucetI div pocetvzorku;
        break;
        end;
  {else v prubehu mereni nekdo prepnul rozsah - musime to zmerit znovu,
        jinak bychom prumerovali jabka s hruskama}
 inc(pojistka); {kdyby nahodou haprovala detekce rozsahu a z portu padala nahodna cisla, je lepsi vcas skoncit}
 until pojistka>100; {za normalnich okolnosti se skonci breakem}
End;{zmerhodnoty}

procedure VypniVystupy; {vypne vsechny vystupni linky portu}
Begin
port[ba+4]:=0; {DTR a RTS}
port[ba+3]:=0; {TXD}
End;{vypnivystupy}

function ZjistiBA(CisloPortu:word):word; {Vrati bazovou adresu serioveho
portu s danym cislem. Pri neuspechu vraci nulu.}
var vysledek:word;
Begin
if (cisloportu>=1) and (cisloportu<=4) then vysledek:=bazoveadresy[cisloportu]
                                       else vysledek:=0;
zjistiba:=vysledek;
End;{zjistiba}

{pomocne graficke procedury:}

procedure Bargraf(x,y,sirka,minimum,maximum,hodnota:integer;
                  BarvaStrafku,BarvaPozadi:byte);
{Vodorovny strafek. X,Y - levy roh, sirka - sirka celeho prvku,
minimum, maximum - meze hodnoty, hodnota - aktualni hodnota k zobrazeni,
barvastrafku - barva leve casti prvku (sirka odpovida hodnote),
barvapozadi - barva zbyle prave casti (mezi hodnotou a maximem)}
const PlnyObdelnicek=#219;
      TeckovanyObdelnicek=#177;
var sirka1,sirka2:word;
Begin
if (minimum=maximum)or(hodnota<=minimum) then begin {tak malo, ze nic neni videt - vsude barva pozadi}
                                              sirka1:=0; sirka2:=sirka;
                                              end
 else if hodnota>=maximum then begin {uplne plno - vsude barva strafku}
                               sirka1:=sirka; sirka2:=0;
                               end
  else begin {nekde mezi - sirka strafku se vyinterpoluje}
       sirka1:=longint(sirka)*(hodnota-minimum) div (maximum-minimum);
       sirka2:=sirka-sirka1;
       end;
writexy(x,y,barvastrafku,kolikceho(sirka1,plnyobdelnicek)); {strafek vlevo}
writexy(x+sirka1,y,barvapozadi,kolikceho(sirka2,teckovanyobdelnicek)); {pozadi vpravo}
End;{bargraf}

procedure Vypis7S(x,y:integer; co:string); {na danych souradnicich (LH roh)
vypise dany retezec (plati jen cislice, tecky a velka pismena, vsechno ostatni
se zobrazi jako mezera)}
const znak=#219; {plny obdelnicek}
      barva=10;
var px:integer;
    i:byte;
Begin
px:=x;
for i:=1 to length(co) do
 if co[i]='.' then begin {desetinna carka}
                   writexy(px,y+4,barva,znak+znak);
                   writexy(px+1,y+5,barva,znak);
                   inc(px,4);
                   end
              else begin {nejaky normalne siroky znak}
                   sedmisegmentovka(px,y,co[i],barva,8,6,3);
                   inc(px,8);
                   end;
End;{vypis7s}

function Zadani(Vyzva,PuvodniText:string; MaxDelka:byte; Typ:byte):string;
{Uprostred obrazovky zobrazi okenko, do ktereho uzivatel zada text.
Po skonceni funkce zustane okenko zobrazene a kurzor vypnuty.
Vyzva - vypise se u horniho okraje okenka.
PuvodniText - predpripravena hodnota v okenku. Vraci se v pripade, ze se
              skonci Escapem nebo neplatnym zadanim.
MaxDelka - do kolika znaku se zadavany text musi vejit.
Typ: 0 - jakykoli text (lze napsat cokoli)
     1 - jmeno souboru (omezena sada znaku, ale smysluplnost se nekontroluje)
     2 - prirozene cislo (jen znaky 0..9)
     3 - cele cislo (0..9 a -)
     4 - realne cislo (0..9, -, tecka a carka (prevede se na tecku))
Navratova hodnota: zadany text. Cisla jsou zkontrolovana a zarucene platna,
                   jmeno souboru je prevedene na velka pismena.}
var i,j,x,y,sirka:integer;
    radek:string;
    o:word;
    znak:char;
    DovoleneZnaky:set of char;
    r:real;
Begin
if typ=0 then dovoleneznaky:=[#0..#255]
         else begin
              dovoleneznaky:=['0'..'9'];
              if typ>=3 then include(dovoleneznaky,'-');
              if typ=4 then dovoleneznaky:=dovoleneznaky+['.',','];
              if typ=1 then dovoleneznaky:=dovoleneznaky+['.','\','A'..'Z','a'..'z',':','-','_','+'];
              end;
{okno:}
radek:='Enter=OK, Esc=zrusit';
sirka:=length(radek);
if length(vyzva)>sirka then sirka:=length(vyzva);
if maxdelka-2>sirka then sirka:=maxdelka-2;
inc(sirka,6);
x:=40-sirka shr 1;
y:=11;
radek:=''+kolikceho(sirka-length(radek)-5,'')+' '+radek+' ͼ';
writexy(x,y+2,$7F,radek);
writexy(x,y,$7F,' '+vyzva+' '+kolikceho(sirka-length(vyzva)-5,'')+'');
writexy(x,y+1,$7F,' '+kolikceho(sirka-4,' ')+' ');
{stin:}
for i:=1 to 2 do cilovatxtobr^[y+i,x+sirka].atribut:=8;
for j:=1 to sirka do cilovatxtobr^[y+3,x+j].atribut:=8;
{hlaska a zadavaci policko:}
tkurzorzap;
radek:=puvodnitext;
i:=length(radek); {pozice kurzoru (za kolikatym znakem je)}
{zadavani:}
 repeat
 writexy(x+2,y+1,$1E,kolikceho(maxdelka,' '));
 writexy(x+2,y+1,$1E,radek);
 gotoxy(x+2+i,y+1);
 o:=xreadkey;
 case o of xlsipka,xhsipka:if i>0 then dec(i);
           xpsipka,xdsipka:if i<length(radek) then inc(i);
           xhome:i:=0;
           xend:i:=length(radek);
           xdel:delete(radek,i+1,1);
           xbksp:if i>0 then begin
                             delete(radek,i,1);
                             dec(i);
                             end;
           xesc:radek:=puvodnitext;
           xins,xpgup,xpgdn,xF1..xF10,xF11,xF12,xenter:{nic, tyhle ignorujeme};
           else begin {napsatelny znak}
                znak:=chr(o);
                if (length(radek)<maxdelka) and (znak in dovoleneznaky)
                  then begin
                       insert(znak,radek,i+1);
                       inc(i);
                       end;
                end;
           end;
 until (o=xenter)or(o=xesc);
tkurzorvyp;
{zpracovani a kontrola vysledku:}
if typ>0 then radek:=stringup(radek);
if typ=4 then for i:=1 to length(radek) do if radek[i]=',' then radek[i]:='.';
case typ of 2:val(radek,o,j);
            3:val(radek,i,j);
            4:val(radek,r,j);
            else j:=0;
            end;
if j=0 then zadani:=radek
       else zadani:=puvodnitext;
End;{zadani}

procedure ChyboveOkno(hlaska:string);
{vykresli danou hlasku v cervenem okne a pocka na stisk klavesy}
var i:word;
Begin
writexy(5,10,$4E,' '+kolikceho(66,'')+' ');
for i:=11 to 13 do writexy(5,i,$4E,' '+kolikceho(66,' ')+' ');
writexy(5,14,$4E,' '+kolikceho(66,'')+' ');
for i:=6 to 75 do cilovatxtobr^[15,i].atribut:=8; {stin dole}
for i:=11 to 14 do cilovatxtobr^[i,75].atribut:=8; {stin vpravo}
writexy(10,11,$4E,hlaska);
writexy(24,13,$30,' Pokracujte libovolnou klavesou. ');
krcr;
End;{chyboveokno}

{dalsi pomocne funkce:}

procedure konfigurak(ulozit:boolean);
{zpracuje konfiguracni soubor: false=nacist, true=ulozit}
var konf:konfigurace;
    bstr:string;
    kod:integer;
Begin
cfgsoubor:='AKUTEST.CFG';
konf.init;
konf.vybersekci('[setup]');
konf.definujpolozku('port number',@cisloportu,_byte);
konf.definujpolozku('base address',@bstr,_string);
konf.definujpolozku('delay 1',@cekaninaprevod,_byte);
konf.definujpolozku('delay 2',@cekaninacs,_byte);
konf.definujpolozku('delay 3',@cekaniclkhigh,_byte);
konf.definujpolozku('delay 4',@cekaniclklow,_byte);
konf.definujpolozku('voltage range A',@rozsahua,_word);
konf.definujpolozku('voltage range B',@rozsahub,_word);
konf.definujpolozku('current range A',@rozsahia,_word);
konf.definujpolozku('current range B',@rozsahib,_word);
konf.definujpolozku('record start voltage',@spoustecinapeti,_real);
konf.definujpolozku('discharge end voltage',@vypinacinapeti,_real);
konf.definujpolozku('recording interval',@intervalzaznamu,_word);
konf.definujpolozku('output file',@jmenozaznamu,_string+79);
konf.definujpolozku('column separator',@oddelovacsloupcu,_char);
konf.definujpolozku('decimal point',@oddelovacdesetin,_char);
konf.definujpolozku('column delimiter',@uvozovka,_string+1);
konf.definujpolozku('number of samples',@pocetvzorku,_byte);
if ulozit then begin
               bstr:='$'+preved(ba,16,3);
               konf.uloz(cfgsoubor);
               end
          else begin
               bstr:='';
               konf.nacti(cfgsoubor);
               if bstr<>'' then begin
                                val(bstr,ba,kod);
                                if kod<>0 then ba:=0;
                                end;
               if pocetvzorku=0 then pocetvzorku:=1
                else if pocetvzorku>100 then pocetvzorku:=100;
               end;
konf.zrus;
End;{konfigurak}

function odchylka(Proud,B:boolean):real;
{Proud - false=napeti, true=proud
 B - false = rozsah A, true = rozsah B
 navratova hodnota - max. chyba 8b prevodu ve V nebo A}
var r:word;
Begin
if proud then if B then r:=rozsahIB
                   else r:=rozsahIA
         else if B then r:=rozsahUB
                   else r:=rozsahUA;
odchylka:=r/256000;
End;{odchylka}

procedure VypisUaI(napeti,proud:real; strafekU,strafekI:byte);
{vypise na obrazovku dane hodnoty napeti a proudu a informace o rozsazich
a presnosti}
var hodnotaU,hodnotaI:string[7];
    barva:byte;
    ch:char;
    r:real;
Begin
str(napeti:6:3,hodnotau);
str(proud:6:3,hodnotai);
{napeti:}
writexy(4,5,7,'Napeti ='); writexy(58,5,7,'V');
writexy(1,9,7,'Vystup ADC:');
writexy(60,3,7,'Rozsah:');
writexy(60,7,7,'Presnost:'); writexy(70,7,10,'+-');
vypis7s(14,3,hodnotau);
writexy(56,9,0,'       ');
if jerozsahnapetib and (napeti*1000<rozsahua) then barva:=14 {v zajmu presnosti se doporucuje prepnout na mensi rozsah}
 else if strafeku=255 then begin {napeti prekrocilo maximum ADC}
                           barva:=12;
                           writexy(56,9,128 or (12 shl 4) or 14,' >MAX! ');
                           end
  else barva:=2; {napeti je v rozumnych mezich}
bargraf(14,9,42,0,255,strafeku,barva,barva);
writexy(72,7,10,nastrr(odchylka(false,jerozsahnapetib),3)+' V ');
if jerozsahnapetib then begin ch:='B'; r:=rozsahub/1000; end
                   else begin ch:='A'; r:=rozsahua/1000; end;
writexy(68,3,10,ch+' ('+nastrr(r,1)+' V)  ');

{proud:}
writexy(5,13,7,'Proud ='); writexy(58,13,7,'A');
writexy(1,17,7,'Vystup ADC:');
writexy(60,11,7,'Rozsah:');
writexy(60,15,7,'Presnost:'); writexy(70,15,10,'+-');
vypis7s(14,11,hodnotai);
writexy(56,17,0,'       ');
{nizky proud nehlasim, prepnutim rozsahu se ztrojnasobi merici odpor a muselo
by se hybat i s velikosti zateze, aby se proud vratil na puvodni hodnotu}
if strafeki=255 then begin {proud prekrocil maximum ADC}
                     barva:=12;
                     writexy(56,17,128 or (12 shl 4) or 14,' >MAX! ');
                     end
                else barva:=2; {proud je v rozumnych mezich}
bargraf(14,17,42,0,255,strafeki,barva,barva);
writexy(72,15,10,nastrr(odchylka(true,jerozsahproudub),3)+' A ');
if jerozsahproudub then begin ch:='B'; r:=rozsahib/1000; end
                   else begin ch:='A'; r:=rozsahia/1000; end;
writexy(68,11,10,ch+' ('+nastrr(r,1)+' A)  ');
End;{vypisuai}

{procedury volane z hlavniho menu:}

procedure VyberPortu;
{Necha uzivatele vybrat port.
Pracuje primo s globalnimi promennymi CisloPortu, BA a Rezim.}
var menu:tmenu;
    b:byte;
Begin
cls(7); gotoxy(3,2);
writeln('Vyberte port, na kterem je pristroj pripojeny:');
writeln('(pozor: autodetekce neni 100% spolehliva, ne vsechny zobrazene porty');
writeln('skutecne fyzicky existuji a funguji)');
menu.init(6);
for b:=1 to 4 do {porty muzou byt max. 4}
 if bazoveadresy[b]<>0
   then menu.polozka('COM'+nastr(b)+' (bazova adresa '+preved(bazoveadresy[b],16,3)+'h)')
   else menu.polozka('-');
menu.polozka('Hlavni menu');
menu.polozka('Konec programu');
if (cisloportu>=1) and (cisloportu<=4) then menu.hodnota:=cisloportu;
 repeat
 menu.kontrola;
 if (menu.zruseno or (menu.hodnota=5)) and (ba<>0) then rezim:=mhlavnimenu
  else if menu.hodnota=6 then rezim:=mkonec
   else if not menu.zruseno then begin
                                 cisloportu:=menu.hodnota;
                                 ba:=zjistiba(cisloportu);
                                 if ba<>0 then rezim:=mhlavnimenu;
                                 end;
 until rezim<>mvyberportu;
End;{vyberportu}

procedure HlavniMenu;
var menu:tmenu;
Begin
cls(7);
writexy(18,2,2,'Merak vybijecich charakteristik akumulatoru');
writexy(17,3,2,kolikceho(45,'='));
writexy(27,4,1,'(cc) Mircosoft 2017..2024');
menu.init(6);
menu.polozka('Rucni rezim');{1}
menu.popis('Na monitoru se zobrazuji okamzite hodnoty napeti a proudu.');
menu.popis('Vybijeni se zapina nebo vypina stiskem klavesy.');
menu.polozka('Nastaveni parametru mereni');{2}
menu.popis('Vsechno, co je potreba pro automaticky rezim: pocatecni a koncove napeti');
menu.popis('a moznosti zaznamu do souboru.');
menu.polozka('Automaticky rezim');{3}
menu.popis('Hlavni funkce programu.');
menu.popis('Napeti, proud a dalsi hodnoty se prubezne zaznamenavaji do souboru.');
menu.popis('Vybijeni se vypina automaticky po poklesu napeti pod nastavenou mez.');
menu.polozka('Vyber portu (aktualne vybrany: COM'+nastr(cisloportu)+')');{4}
menu.popis('Prepnuti na jiny port pro ovladani pristroje.');
menu.polozka('Kalibrace');{5}
menu.popis('Urceni korekcniho soucinitele pro prevod signalu z A/D prevodniku na volty');
menu.popis('a ampery, nezbytne pro spravnou funkci pristroje. Kalibraci staci provest');
menu.popis('jednou pred prvnim pouzitim.');
menu.polozka('Konec');{6}
menu.popis('Ukonci program. Nebo staci zmacknout Esc, vyjde to nastejno.');
menu.kontrola;
if menu.zruseno then rezim:=mkonec
                else case menu.hodnota of 1:rezim:=mRucniMereni;
                                          2:rezim:=mNastaveni;
                                          3:rezim:=mAutomatickeMereni;
                                          4:rezim:=mVyberPortu;
                                          5:rezim:=mkalibrace;
                                          6:rezim:=mkonec;
                                          end;
End;{hlavnimenu}

procedure RucniMereni;
var napeti,proud,r:real;
    strafekU,strafekI,cas1,cas2:byte;
    ReleZapnuto:boolean;
    casovac:byte absolute 0:$046C; {ve skutecnosti dword, ale zajimaji nas jenom zmeny nejnizsiho bytu}
    o:char;
Begin
cls(7);
writexy(31,1,14,'=== Rucni rezim ===');
writexy(18,25,3 shl 4,' Pro navrat do hlavniho menu stisknete Esc. ');

writexy(4,20,7,'Vybijeni:'); writexy(35,20,3 shl 4,' (prepinejte Enterem) ');
relezapnuto:=false;
writexy(1,22,7,'Rozsahy merenych velicin prepinejte primo na pristroji.');
writexy(1,23,7,'Prouzky "Vystup ADC" ukazuji, v jake casti rozsahu prevodnik pracuje.');

 repeat
 cas1:=casovac;
 zmerhodnoty(napeti,proud,strafeku,strafeki);
 {vypis hodnot:}
 vypisuai(napeti,proud,strafeku,strafeki);
 {rele:}
 if relezapnuto then writexy(14,20,2 shl 4 or 15,' +++  Zapnuto  +++ ')
                else writexy(14,20,4 shl 4 or 15,' ---  Vypnuto  --- ');
 {cekani na vstup od uzivatele nebo uplynuti 55 ms:}
  repeat
  cas2:=casovac;
  until (cas2<>cas1) or keypressed;
 if keypressed then begin
                    o:=readkey;
                    if o=#13 then begin
                                  relezapnuto:=not relezapnuto;
                                  rele(relezapnuto);
                                  end;
                    end
               else o:=#0;

 until o=#27;
kresetuj;
rele(false);
rezim:=mhlavnimenu;
End;{rucnimereni}

procedure NastaveniAutomatu;
var menu:tmenu;
    zadano:string[80];
    w:word;
    i:integer;
Begin
cls(7);
writexy(24,2,7,'Nastaveni parametru mereni');
writexy(17,3,7,kolikceho(41,'-'));
menu.init(5);
menu.polozka('Spustit zaznam pri poklesu napeti pod: '+nastrr(spoustecinapeti,3)+' V');{1}
menu.popis('Urceno pro zajisteni stejnych pocatecnich podminek pri mereni vice');
menu.popis('akumulatoru. Vybijeni spustite rucne, ale pocitani kapacity a zaznam do');
menu.popis('souboru zacne az po dosazeni tohoto napeti. Jestli chcete zaznamenavat hned,');
menu.popis('zadejte hodnotu vetsi nez pocatecni napeti akumulatoru.');
menu.polozka('Vypnout vybijeni pri poklesu napeti pod: '+nastrr(vypinacinapeti,3)+' V');{2}
menu.popis('Velmi dulezita vec. Vzdy nastavte podle toho, jaky akumulator merite!');
menu.popis('Zarucene bezpecna hodnota pro Li-ion by mela byt 3 V na clanek, pro NiCd a');
menu.popis('NiMH 1 V/clanek. Presna cisla by mela byt k nalezeni v navodu od vyrobce');
menu.popis('konkretniho akumulatoru.');
menu.polozka('Prubehy velicin zaznamenavat do souboru: '+jmenozaznamu);{3}
menu.popis('Zadejte jmeno souboru i s koncovkou (nejlepe .CSV). Jestli ho chcete');
menu.popis('dat jinam nez do adresare s programem, zadejte i cestu (adresare oddelujte');
menu.popis('znakem "\").');
menu.popis('Pozor: soubor se s kazdym novym merenim prepisuje. Za zacatek noveho mereni');
menu.popis('se povazuje prechod z hlavniho menu na obrazovku automatickeho rezimu.');
menu.polozka('Oddelovac sloupcu: '+oddelovacsloupcu);{4}
menu.popis('Data se do souboru zapisuji ve forme tabulky: prvni radek je hlavicka s');
menu.popis('nazvy sloupcu, kazdy dalsi radek je nekolik oddelenych ciselnych hodnot.');
menu.popis('Mezi jednotlivymi hodnotami je tento oddelovaci znak. Pro format CSV se');
menu.popis('pouziva carka (,), ale nektere tabulkove editory (napr. Excel od rocniku');
menu.popis('2007 dal) vyzaduji strednik (;).');
menu.polozka('Desetinna carka: '+oddelovacdesetin);{5}
menu.popis('Standardne se pouziva desetinna tecka (.), ale nektere ceske verze Excelu');
menu.popis('vyzaduji carku (,). Pozor, ze v takovem pripade je nutne pouzit jiny');
menu.popis('oddelovac sloupcu nebo ohraniceni, aby se to nepletlo.');
menu.polozka('Ohraniceni sloupcu: '+uvozovka);{6}
menu.popis('Nutne pouze pokud pouzivate na oddeleni desetin i sloupcu stejny znak.');
menu.popis('Obvykle se pouziva uvozovka (").');
menu.polozka('Zapisovat kazdych: '+nastr(intervalzaznamu)+' s');{7}
menu.popis('Po kolika vterinach se ma do souboru ulozit dalsi radek zaznamu.');
menu.popis('Nema to vliv na frekvenci mereni, kontrol a vypoctu, ty probihaji vzdy');
menu.popis('po 1 vterine.');
menu.polozka('Zpet na hlavni menu');{8}
 repeat
 menu.kontrola;
 if menu.zruseno
   then rezim:=mhlavnimenu
   else case menu.hodnota of
          1:begin
            spoustecinapeti:=valfr(zadani('Spousteci napeti:',nastrr(spoustecinapeti,4),10,4),spoustecinapeti);
            menu.prepispolozku(1,'Spustit zaznam pri poklesu napeti pod: '+nastrr(spoustecinapeti,3)+' V');
            end;
          2:begin
            vypinacinapeti:=valfr(zadani('Vypinaci napeti:',nastrr(vypinacinapeti,4),10,4),vypinacinapeti);
            menu.prepispolozku(2,'Vypnout vybijeni pri poklesu napeti pod: '+nastrr(vypinacinapeti,3)+' V');
            end;
          3:begin
            jmenozaznamu:=zadani('Jmeno souboru:',jmenozaznamu,60,1);
            menu.prepispolozku(3,'Prubehy velicin zaznamenavat do souboru: '+jmenozaznamu);
            end;
          4:begin
            zadano:=zadani('Oddelovac sloupcu:',oddelovacsloupcu,1,0);
            if length(zadano)=1 then OddelovacSloupcu:=zadano[1];
            menu.prepispolozku(4,'Oddelovac sloupcu: '+oddelovacsloupcu);
            end;
          5:begin
            zadano:=zadani('Desetinna carka:',oddelovacdesetin,1,0);
            if length(zadano)=1 then oddelovacdesetin:=zadano[1];
            menu.prepispolozku(5,'Desetinna carka: '+oddelovacdesetin);
            end;
          6:begin
            zadano:=zadani('Ohraniceni sloupcu:',uvozovka,1,0);
            if length(zadano)<=1 then uvozovka:=zadano;
            menu.prepispolozku(6,'Ohraniceni sloupcu: '+uvozovka);
            end;
          7:begin
            zadano:=zadani('Interval zaznamu:',nastr(intervalzaznamu),5,2);
            val(zadano,w,i);
            if i=0 then intervalzaznamu:=w;
            menu.prepispolozku(7,'Zapisovat kazdych: '+nastr(intervalzaznamu)+' s');
            end;
          8:rezim:=mhlavnimenu;
          end;
 until rezim<>mnastaveni;
End;{nastaveniautomatu}

procedure AutomatickeMereni;
var napeti,proud,r:real;
    strafekU,strafekI,barva:byte;
    nic:word;
    o:char;
    ReleZapnuto:boolean;
    rok,mesic,den,hodiny,minuty,sekundy1,sekundy2,tmp:word;
    mAhA,mAhB:comp; {soucty syrovych vzorku proudu pro rozsahy A a B (longint nestaci)}
    mAh:real; {celkova prepocitana hodnota vycerpane kapacity}
    Merime,MeriliJsme:boolean;
    ZacatekVybijeni,KonecVybijeni,l:longint; {[s]}
    DobaVybijeni:word; {[s]}
    UplynuleVteriny:word; {pro pocitani intervalu zaznamu}
    soubor:text;
    radek:string;
Begin
{priprava souboru:}
assign(soubor,jmenozaznamu);
rewrite(soubor);
writeln(soubor,uvozovka,'Doba vybijeni [s]',uvozovka,oddelovacsloupcu,
               uvozovka,'Kapacita [mAh]',uvozovka,oddelovacsloupcu,
               uvozovka,'Napeti [V]',uvozovka,oddelovacsloupcu,
               uvozovka,'Proud [A]',uvozovka);
close(soubor);
if ioresult<>0
  then begin
       chyboveokno('Nelze otevrit soubor '+jmenozaznamu);
       rezim:=mhlavnimenu;
       exit;
       end;
{konstantni texty:}
cls(7);
writexy(26,1,14,'=== Automaticky rezim ===');
writexy(18,25,$30,' Pro navrat do hlavniho menu stisknete Esc. ');
writexy(4,20,7,'Vybijeni:'); writexy(35,20,3 shl 4,' (prepinejte Enterem) ');
writexy(1,22,7,'Doba vybijeni:');
writexy(40,22,7,'Vycerpana kapacita:'); writexy(69,22,10,'mAh');
writexy(1,23,7,'Spousteci napeti: '+nastrr(spoustecinapeti,1)+' V');
writexy(40,23,7,'Vypinaci napeti: '+nastrr(vypinacinapeti,1)+' V');

relezapnuto:=false;
maha:=0; mahb:=0;
dobavybijeni:=0;
merilijsme:=false;
uplynulevteriny:=1; {aby se hned po spusteni ulozilo}
{inicializace zacatku a konce:}
getdate(rok,mesic,den,tmp); gettime(hodiny,minuty,sekundy1,tmp);
l:=((((longint(rok-1980)*12+mesic)*31+den)*24+hodiny)*60+minuty)*60+sekundy1;
zacatekvybijeni:=l; konecvybijeni:=l;

 repeat
 getdate(rok,mesic,den,tmp);
 gettime(hodiny,minuty,sekundy1,tmp);
 writexy(73,1,4,nastr2(hodiny,2)+':'+nastr3(minuty,2)+':'+nastr3(sekundy1,2));
 zmerhodnoty(napeti,proud,strafeku,strafeki);
 relezapnuto:=relezapnuto and (napeti>vypinacinapeti);
 rele(relezapnuto);
 vypisuai(napeti,proud,strafeku,strafeki);

 {doba vybijeni:}
 if merilijsme and not merime {skoncil usek, kdy se merilo - ulozime jeho delku}
   then inc(dobavybijeni,konecvybijeni-zacatekvybijeni);
 merilijsme:=merime;
 l:=((((longint(rok-1980)*12+mesic)*31+den)*24+hodiny)*60+minuty)*60+sekundy1;
 if merime then konecvybijeni:=l else zacatekvybijeni:=l;
 if merime then l:=dobavybijeni+konecvybijeni-zacatekvybijeni
           else l:=dobavybijeni;
 writexy(16,22,10,nastr2(l div 3600,2)+':'+nastr3((l mod 3600)div 60,2)
                  +':'+nastr3(l mod 60,2)); {pozor: l se jeste vyuzije pri zaznamu}

 {vycerpana kapacita:}
 if merime then if jerozsahprouduB then mahb:=mahb+strafeki {comp se chova jako realne cislo, inc nejde}
                                   else maha:=maha+strafeki;
 mah:=((mahA*rozsahIA)+(mahB*rozsahIB))/(256*3600);
  {Pozn.: kdyby byly promenne mahA a mahB longinty, vyraz na predchozim radku
  by pri vyslednych hodnotach kolem 2300 mAh pretekal. Upravou na dva zlomky
  by sel rozsah zvetsit na dvojnasobek, prekompilovanim v prekladaci, ktery
  zvlada dword, na dalsi dvojnasobek (celkem max. 9200 mAh). To ale porad
  nemusi stacit, tak jsem pro jistotu radsi pouzil 64b typ comp.}
 writexy(60,22,10,zarovnej(nastrr(mah,1),8,_doprava,' '));

 {vypinaci napeti:}
 if napeti>vypinacinapeti then tmp:=7 {dokud jsme nad vypinacim napetim, hodnota zelene sviti}
                          else tmp:=12+128; {kdyz klesneme pod nej, hodnota cervene blika}
 writexy(57,23,tmp,nastrr(vypinacinapeti,1)+' V');

 {zaznam do souboru:}
 writexy(61,19,7,kolikceho(14,' ')); {smazani minule hlasky "ulozeno"}
 if merime
   then begin
        writexy(59,18,128+$2A,'   Probiha mereni  ');
        dec(uplynulevteriny);
        if (uplynulevteriny=0) or not relezapnuto {je cas ukladat: uplynul zadany interval nebo se prave vyplo rele}
          then begin
               uplynulevteriny:=intervalzaznamu;
               radek:=uvozovka+nastr(l)+uvozovka+oddelovacsloupcu {doba vybijeni}
                      +uvozovka+nastrr(mah,1)+uvozovka+oddelovacsloupcu
                      +uvozovka+nastrr(napeti,3)+uvozovka+oddelovacsloupcu
                      +uvozovka+nastrr(proud,3)+uvozovka;
               if oddelovacdesetin<>'.'
                 then for tmp:=1 to length(radek) do
                       if radek[tmp]='.' then radek[tmp]:=oddelovacdesetin;
               append(soubor);
               writeln(soubor,radek);
               close(soubor);
               if ioresult=0 then writexy(65,19,10,'Ulozeno')
                             else writexy(61,19,$4E,'Chyba ukladani');
               end;
        end
   else writexy(59,18,$40,'  Mereni vypnute   ');
 merime:=(napeti<=spoustecinapeti) and relezapnuto;
 {kontrolka rele:}
 if relezapnuto then writexy(14,20,2 shl 4 or 15,' +++  Zapnuto  +++ ')
                else writexy(14,20,4 shl 4 or 15,' ---  Vypnuto  --- ');

 {cekani na vstup od uzivatele nebo uplynuti 1 s:}
  repeat
  gettime(hodiny,minuty,sekundy2,tmp);
  until (sekundy2<>sekundy1) or keypressed;
 if keypressed then begin
                    o:=readkey;
                    if o=#13 then begin
                                  relezapnuto:=not relezapnuto;
                                  rele(relezapnuto);
                                  {vterinu pockame na precvaknuti rele:}
                                  gettime(hodiny,minuty,sekundy1,tmp);
                                   repeat
                                   gettime(hodiny,minuty,sekundy2,tmp);
                                   until sekundy2<>sekundy1;
                                  end;
                    end
               else o:=#0;
 until o=#27;
kresetuj;
rele(false);
rezim:=mhlavnimenu;
End;{automatickemereni}

procedure Kalibrace;
var napeti,proud,r:real;
    strafekU,strafekI,cas1,cas2:byte;
    ReleZapnuto,JedemeDal:boolean;
    casovac:byte absolute 0:$046C; {ve skutecnosti dword, ale zajimaji nas jenom zmeny nejnizsiho bytu}
    o:char;
    zadano:string[10];
    kod:integer;
Begin
relezapnuto:=false;
 repeat
 jedemedal:=false;
 cls(7);
 writexy(32,1,14,'=== Kalibrace ===');
 writexy(18,25,3 shl 4,' Pro navrat do hlavniho menu stisknete Esc. ');

 writexy(4,19,7,'Vybijeni:'); writexy(35,19,3 shl 4,' (prepinejte Enterem) ');
 writexy(1,21,7,'Na svorky pristroje privedte kalibracni napeti nebo proud. Velikost volte');
 writexy(1,22,7,'pokud mozno tak, aby prislusny vystup ADC byl tesne u horniho konce rozsahu.');
 writexy(2,23,$30,' U ');
 writexy(6,23,7,'- zadani hodnoty kalibracniho napeti');
 writexy(2,24,$30,' I ');
 writexy(6,24,7,'- zadani hodnoty kalibracniho proudu');

  repeat
  cas1:=casovac;
  zmerhodnoty(napeti,proud,strafeku,strafeki);
  {vypis hodnot:}
  vypisuai(napeti,proud,strafeku,strafeki);
  {rele:}
  if relezapnuto then writexy(14,19,2 shl 4 or 15,' +++  Zapnuto  +++ ')
                 else writexy(14,19,4 shl 4 or 15,' ---  Vypnuto  --- ');
  {cekani na vstup od uzivatele nebo uplynuti 55 ms:}
   repeat
   cas2:=casovac;
   until (cas2<>cas1) or keypressed;
  if keypressed
    then begin
         o:=readkey;
         case o of #13:begin
                       relezapnuto:=not relezapnuto;
                       rele(relezapnuto);
                       end;
                   'u','U':begin
                           if strafeku=0 then chyboveokno('Nelze kalibrovat nulovym napetim.')
                            else if strafeku=255 then chyboveokno('Napeti je mimo rozsah prevodniku.')
                             else begin
                                  zadano:=zadani('Zadejte presnou hodnotu napeti:',nastrr(napeti,3),10,4);
                                  val(zadano,r,kod); {kod je zbytecny, urcite to vyjde}
                                  r:=256000*r/strafeku;
                                  if jerozsahnapetib then rozsahub:=round(r)
                                                     else rozsahua:=round(r);
                                  end;
                           o:=#27;          {\ at se prekresli}
                           jedemedal:=true; {/ cela obrazovka }
                           end;
                   'i','I':begin
                           if strafeki=0 then chyboveokno('Nelze kalibrovat nulovym proudem.')
                            else if strafeki=255 then chyboveokno('Proud je mimo rozsah prevodniku.')
                             else begin
                                  zadano:=zadani('Zadejte presnou hodnotu proudu:',nastrr(proud,3),10,4);
                                  val(zadano,r,kod);
                                  r:=256000*r/strafeki;
                                  if jerozsahproudub then rozsahib:=round(r)
                                                     else rozsahia:=round(r);
                                  end;
                           o:=#27;
                           jedemedal:=true;
                           end;
                   end;
         end
    else o:=#0;
  until o=#27;
 until not jedemedal;
kresetuj;
rele(false);
rezim:=mhlavnimenu;
End;{kalibrace}

BEGIN
nastavtextak80x25; tkurzorvyp;
konfigurak(false);
if (cisloportu<1)or(cisloportu>4) then cisloportu:=0;
if cisloportu=0 then ba:=0;
if ba=0 then ba:=zjistiba(cisloportu);
if ba=0 then rezim:=mvyberportu
        else rezim:=mhlavnimenu;
vypnivystupy;
 repeat
 case rezim of
  mhlavnimenu:hlavnimenu;
  mVyberPortu:vyberportu;
  mkalibrace:kalibrace;
  mRucniMereni:rucnimereni;
  mAutomatickeMereni:automatickemereni;
  mNastaveni:nastaveniautomatu;
  else rezim:=mkonec;
  end;
 until rezim=mkonec;
{zaverecny uklid:}
konfigurak(true);
vypnivystupy;
cls(7); gotoxy(1,1); tkurzorzap;
END.

{Schema zapojeni:

   PC                   Pripravek
pin  signal    smer    pin    signal
COM                    ADC
 1    DCD   <--------         indikace rozsahu B pro ampermetr
 2    RXD   nezapojeno
 3    TXD   -------->         signal pro sepnuti rele
 4    DTR   -------->  5      Chip Selection pro oba prevodniky
 5    SG    <------->  3, 4   GND, Ref-, spolecna zem pro vsechno
 6    DSR   <--------  6      Data Out pro prevodnik ampermetru
 7    RTS   -------->  7      I/O Clock pro oba prevodniky
 8    CTS   <--------  6      Data Out pro prevodnik voltmetru
 9    RI    <--------         indikace rozsahu B pro voltmetr
napajeci zdroj:
    +12 V   -------->         napajeni civky rele
     +5 V   -------->  1, 8   Ucc, Ref+ pro oba prevodniky a pomocne signaly

 Urovne napeti COM jsou oficialne -3..-15 V pro L a +3..+15 V pro H, mereno
vzhledem k vodici SG. Prakticky je L cokoli pod 3 V, vcetne nepripojeneho
vodice.

Doporucene napajeci napeti TLC549 je 5 V, doporuceny rozsah mereneho
analogoveho napeti je 0..5 V. Urovne napeti pro ridici vstupy jsou max. 0.8 V
pro L a min. 2 V pro H. Vystupy davaji max. 0.4 V pri L a min 2.4 V pri H
(nameril jsem neco kolem 4.5 V, na seriovy port to staci i bez zesilovacu).}
